'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createValidator;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _lodash = require('lodash.isplainobject');

var _lodash2 = _interopRequireDefault(_lodash);

var _configureValueValidator = require('./configureValueValidator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createValidator(curriedDefinition, defaultMessageCreator) {
  var messageCreatorIsString = typeof defaultMessageCreator === 'string';

  (0, _invariant2.default)(messageCreatorIsString || typeof defaultMessageCreator === 'function', 'Please provide a message string or message creator function');

  return function validator(config, value) {
    var configIsObject = (0, _lodash2.default)(config);

    if (!messageCreatorIsString) {
      (0, _invariant2.default)(typeof config === 'string' || configIsObject, 'Please provide a string or configuration object with a `field` or ' + '`message` property');

      if (configIsObject) {
        (0, _invariant2.default)('field' in config || 'message' in config, 'Please provide a `field` or `message` property');
      }
    }

    var message = configIsObject && 'message' in config ? config.message : messageCreatorIsString ? defaultMessageCreator : configIsObject ? defaultMessageCreator(config.field) : defaultMessageCreator(config);

    var valueValidator = curriedDefinition(message);

    if (arguments.length <= 1) {
      return (0, _configureValueValidator.markAsValueValidator)(valueValidator);
    }

    return valueValidator(value);
  };
} /* eslint semi:0 */